!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center
! Land Information System Framework (LISF)
! Version 7.3
!
! Copyright (c) 2020 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LIS_misc.h"
!BOP
!
! !ROUTINE: template_main
! \label{template_main}
!
! !ROUTINE: template_main.F90
!    Apr 2003; Sujay Kumar, Initial Code
! 23 Oct 2007; Kristi Arsenault, Updated code for LISv50
! 
! !INTERFACE:
subroutine template_main(n)

  use LIS_coreMod,       only : LIS_rc, LIS_domain, LIS_localPet
  use LIS_histDataMod

  implicit none
! !ARGUMENTS: 
  integer, intent(in)  :: n
!
! !DESCRIPTION:
! 
!  Calls the run routines for the forcing-only option (template)
!
!  The arguments are: 
!  \begin{description}
!  \item[n]
!   index of the nest
!  \end{description}
!EOP

  integer :: t, ts
  real, dimension(8,LIS_rc%ntiles(n)) :: array

  ts = LIS_rc%tscount(n)
  if ( ts == 1 ) then
     do t = 1, LIS_rc%ntiles(n)
        print*,"grid", LIS_domain(n)%tile(t)%index, "tile ",t,&
               " fgrd = ",LIS_domain(n)%tile(t)%fgrd,"(",LIS_localPet,")"
        print*,"grid", LIS_domain(n)%tile(t)%index, "tile ",t,&
               " pens = ",LIS_domain(n)%tile(t)%pens,"(",LIS_localPet,")"
     enddo
  endif
!  print*,'GREP: time = ',ts

#if 0
  !-----------------------------------------------------------------------
  !3x3; 1 tile per grid; 1 ensemble per tile
  !-----------------------------------------------------------------------
  ! grid           1 tile            1  fgrd =    1.000000
  ! grid           1 tile            1  pens =    1.000000
  ! grid           2 tile            2  fgrd =    1.000000
  ! grid           2 tile            2  pens =    1.000000
  ! grid           3 tile            3  fgrd =    1.000000
  ! grid           3 tile            3  pens =    1.000000
  ! grid           4 tile            4  fgrd =    1.000000
  ! grid           4 tile            4  pens =    1.000000
  ! grid           5 tile            5  fgrd =    1.000000
  ! grid           5 tile            5  pens =    1.000000
  ! grid           6 tile            6  fgrd =    1.000000
  ! grid           6 tile            6  pens =    1.000000
  ! grid           7 tile            7  fgrd =    1.000000
  ! grid           7 tile            7  pens =    1.000000
  ! grid           8 tile            8  fgrd =    1.000000
  ! grid           8 tile            8  pens =    1.000000
  ! grid           9 tile            9  fgrd =    1.000000
  ! grid           9 tile            9  pens =    1.000000

  !First output at min 15
  array(1,:) = (/4.716996, -5.8574688E-02, -4.946619, -4.528573, 4.050246, 0.3562459, 1.049067, 9.616941, 9.012380/)

  !Second output at hour 1
  array(2,:) = (/6.697559, -3.812158, 7.377232, -6.422610, -3.549520, -4.753593, -7.389554, -4.095519, 9.354531/)
  array(3,:) = (/-7.382446, -1.403471, -7.977238, -5.001271, 5.461119, -6.588569, -4.501272, -4.737327, -1.895595/)
  array(4,:) = (/-9.781530, 9.290967, 8.008259, -3.773006, -2.737287, 2.721999, 8.338540, 1.821654, -3.868184/)
  !instantaneous: -9.781530       9.290967       8.008259      -3.773006      -2.737287    2.721999       8.338540       1.821654      -3.868184
  !accumulated:  -10.46642       4.075338       7.408253      -15.19689     -0.8256879    -8.620163      -3.552286      -7.011192       3.590752
  !averaged:    -3.488806       1.358446       2.469417      -5.065629     -0.2752293    -2.873388      -1.184095      -2.337064       1.196917
  !min:           -9.781530    -3.812158      -7.977238      -6.422610     -3.549520     -6.588569      -7.389554      -4.737327      -3.868184
  !max:            6.697559     9.290967      8.008259       -3.773006      5.461119      2.721999       8.338540       1.821654       9.354531
  !inst mean:    -9.781530       9.290967       8.008259      -3.773006      -2.737287    2.721999       8.338540       1.821654      -3.868184    
  !inst stdev:   0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00 0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00
  !accum mean:   -10.46642       4.075338       7.408253      -15.19689     -0.8256879    -8.620163      -3.552286      -7.011192       3.590752    
  !accum stdev:  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00 0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00
  !avg mean:     -3.488806       1.358446       2.469417      -5.065629     -0.2752293    -2.873388      -1.184095      -2.337064       1.196917    
  !avg stdev:   0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00 0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00

  !Third output at hour 2
  array(5,:) = (/3.893483, -5.318944, -3.282447, 4.280159, 0.6406029, -5.399737, 9.492588, 2.222070, -3.001371/)
  array(6,:) = (/2.292842, 1.003245, 3.493016, 8.131827, 0.7645558, -3.0272238E-02, -4.130377, -4.099755, 3.070868/)
  array(7,:) = (/7.338583, 4.381160, -8.673751, 6.1191890E-02, -5.017724, -7.243008, -6.492382, -1.514535, -3.883759/)
  array(8,:) = (/3.083600, -6.318086, -5.699180, -1.090655, 0.3126619, -1.287766, -9.275128, -2.084915, -4.124979/)
  !instantaneous: 3.083600      -6.318086      -5.699180      -1.090655      0.3126619    -1.287766      -9.275128      -2.084915      -4.124979
  !accumulated:   16.60851      -6.252625      -14.16236       11.38252      -3.299903    -13.96078      -10.40530      -5.477135      -7.939241
  !averaged:      4.152127      -1.563156      -3.540591       2.845631     -0.8249758    -3.490196      -2.601325      -1.369284      -1.984810
  !min:           2.292842       -6.318086     -8.673751       -1.090655    -5.017724     -7.243008      -9.275128      -4.099755      -4.124979
  !max:           7.338583      4.381160       3.493016       8.131827       0.7645558    -3.0272238E-02, 9.492588       2.222070       3.070868
  !inst mean:     3.083600      -6.318086      -5.699180      -1.090655      0.3126619    -1.287766      -9.275128      -2.084915      -4.124979    
  !inst stdev:   0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00 0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00
  !accum mean:    16.60851      -6.252625      -14.16236       11.38252      -3.299903    -13.96078      -10.40530      -5.477135      -7.939241    
  !accum stdev:   0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00 0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00
  !avg mean:      4.152127      -1.563156      -3.540591       2.845631     -0.8249758    -3.490196      -2.601325      -1.369284      -1.984810    
  !avg stdev:    0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00 0.0000000E+00  0.0000000E+00  0.0000000E+00  0.0000000E+00
#endif

#if 0
  !-----------------------------------------------------------------------
  !3x3; 1 tile per grid; 2 ensembles per tile
  !-----------------------------------------------------------------------
  ! grid           1 tile            1  fgrd =    1.000000    
  ! grid           1 tile            1  pens =   0.5000000    
  ! grid           1 tile            2  fgrd =    1.000000    
  ! grid           1 tile            2  pens =   0.5000000    
  ! grid           2 tile            3  fgrd =    1.000000    
  ! grid           2 tile            3  pens =   0.5000000    
  ! grid           2 tile            4  fgrd =    1.000000    
  ! grid           2 tile            4  pens =   0.5000000    
  ! grid           3 tile            5  fgrd =    1.000000    
  ! grid           3 tile            5  pens =   0.5000000    
  ! grid           3 tile            6  fgrd =    1.000000    
  ! grid           3 tile            6  pens =   0.5000000    
  ! grid           4 tile            7  fgrd =    1.000000    
  ! grid           4 tile            7  pens =   0.5000000    
  ! grid           4 tile            8  fgrd =    1.000000    
  ! grid           4 tile            8  pens =   0.5000000    
  ! grid           5 tile            9  fgrd =    1.000000    
  ! grid           5 tile            9  pens =   0.5000000    
  ! grid           5 tile           10  fgrd =    1.000000    
  ! grid           5 tile           10  pens =   0.5000000    
  ! grid           6 tile           11  fgrd =    1.000000    
  ! grid           6 tile           11  pens =   0.5000000    
  ! grid           6 tile           12  fgrd =    1.000000    
  ! grid           6 tile           12  pens =   0.5000000    
  ! grid           7 tile           13  fgrd =    1.000000    
  ! grid           7 tile           13  pens =   0.5000000    
  ! grid           7 tile           14  fgrd =    1.000000    
  ! grid           7 tile           14  pens =   0.5000000    
  ! grid           8 tile           15  fgrd =    1.000000    
  ! grid           8 tile           15  pens =   0.5000000    
  ! grid           8 tile           16  fgrd =    1.000000    
  ! grid           8 tile           16  pens =   0.5000000    
  ! grid           9 tile           17  fgrd =    1.000000    
  ! grid           9 tile           17  pens =   0.5000000    
  ! grid           9 tile           18  fgrd =    1.000000    
  ! grid           9 tile           18  pens =   0.5000000    

  !First output at min 15
  array(1,:)=(/4.716996, -5.8574688E-02, -4.946619, -4.528573, 4.050246, 0.3562459, 1.049067, 9.616941, 9.012380, 9.418849, 9.940899, 6.408007, -1.736115, 5.853973, -7.550773, -2.725465, 1.651574, 6.570506/)

  !Second output at hour 1
  array(2,:)=(/6.697559, -3.812158, 7.377232, -6.422610, -3.549520, -4.753593, -7.389554, -4.095519, 9.354531, -2.984151, 2.352701, -6.535456, 0.7984526, -8.943732, 6.086356, -2.954750, -4.079035, -6.946579/)
  array(3,:)=(/-7.382446, -1.403471, -7.977238, -5.001271, 5.461119, -6.588569, -4.501272, -4.737327, -1.895595, -2.482169, 1.397863, 0.5272363, -8.027037, 9.894200, -4.217344, -7.860428, -3.925225, 6.145012/)
  array(4,:)=(/-9.781530, 9.290967, 8.008259, -3.773006, -2.737287, 2.721999, 8.338540, 1.821654, -3.868184, -2.530884, 6.939022, -9.691016, 4.223316, 9.007157, 9.105985, -8.339506, 6.217335, -7.065415/)
  !instantaneous: -0.2452817       2.117626     -7.6440573E-03   5.080097      -3.199534    -1.375997       6.615237      0.3832393     -0.4240398
  !accumulated:   -3.195540      -3.894317      -4.722926      -5.281739      -2.203226    -2.504825       3.476178      -4.089843      -4.826953
  !averaged:      -1.065180      -1.298106      -1.574309      -1.760580     -0.7344086    -0.8349416       1.158726      -1.363281      -1.608984
  !min:           -9.781530      -7.977238      -6.588569      -7.389554      -3.868184     -9.691016      -8.943732      -8.339506      -7.065415
  !max:            9.290967       8.008259       5.461119       8.338540       9.354531      6.939022        9.894200       9.105985       6.217335
  !inst mean:     -0.2452817       2.117626     -7.6440573E-03   5.080097      -3.199534    -1.375997       6.615237      0.3832393     -0.4240398    
  !inst stdev:     9.536248       5.890633       2.729643       3.258443      0.6686500    8.315020       2.391921       8.722746       6.641375    
  !accum mean:    -3.195540      -3.894317      -4.722925      -5.281739      -2.203226    -2.504825       3.476179      -4.089844      -4.826953    
  !accum stdev:    7.270878       11.30257       3.897238       1.729453       5.793978    13.19441       6.481446       15.06484       3.040029    
  !avg mean:      -1.065180      -1.298106      -1.574308      -1.760580     -0.7344086    -0.8349416       1.158726      -1.363281      -1.608984    
  !avg stdev:      2.423626       3.767523       1.299079      0.5764842       1.931326    4.398137       2.160482       5.021613       1.013343    

  !Third output at hour 2
  array(5,:)=(/3.893483, -5.318944, -3.282447, 4.280159, 0.6406029, -5.399737, 9.492588, 2.222070, -3.001371, -8.529298, 9.347016, -8.244946, 8.028025, 1.886872, -2.056841, -5.433362, -2.393033, 0.5268470/)
  array(6,:)=(/2.292842, 1.003245, 3.493016, 8.131827, 0.7645558, -3.0272238E-02, -4.130377, -4.099755, 3.070868, -9.546155, -9.099010, -3.588018, -7.537397, 1.582848, 8.247005, 2.512011, -5.205674, -4.737566/)
  array(7,:)=(/7.338583, 4.381160, -8.673751, 6.1191890E-02, -5.017724, -7.243008, -6.492382, -1.514535, -3.883759, 2.662015, 7.436269, 1.580206, -5.054608, 9.078156, -1.479063, 6.945909, 8.579399, -6.499947/)
  array(8,:)=(/3.083600, -6.318086, -5.699180, -1.090655, 0.3126619, -1.287766, -9.275128, -2.084915, -4.124979, 9.457637, 9.837383, -1.176786, -7.695197, 6.274336, 6.044065, -4.654425, -7.474986, -5.760541/)
  !instantaneous: -1.617243      -3.394917     -0.4875520      -5.680022       2.666329    4.330298     -0.7104306      0.6948199      -6.617764
  !accumulated:   5.177941      -1.389920      -8.630342      -7.941217      -6.947521    3.046057       3.281518       5.062650      -11.48275
  !averaged:     1.294485     -0.3474799      -2.157586      -1.985304      -1.736880    0.7615144      0.8203794       1.265662      -2.870687
  !min:          -6.318086     -8.673751       -7.243008      -9.275128      -9.546155   -9.099010     -7.695197       -5.433362      -7.474986
  !max:           7.338583     8.131827        0.7645558      9.492588        9.457637    9.837383      9.078156        8.247005       8.579399
  !inst mean:    -1.617243      -3.394917     -0.4875520      -5.680022       2.666329    4.330298     -0.7104306      0.6948199      -6.617764    
  !inst stdev:    4.700843       2.304263      0.8002139       3.595106       6.791308    5.507084       6.984767       5.349245      0.8572226    
  !accum mean:    5.177942      -1.389920      -8.630343      -7.941217      -6.947521    3.046057       3.281518       5.062650      -11.48275    
  !accum stdev:   11.43057       12.77244       5.330440       2.464082      0.9917200    14.47560       15.54070       5.692516       4.988456    
  !avg mean:       1.294485     -0.3474801      -2.157586      -1.985304      -1.736880    0.7615142      0.8203794       1.265662      -2.870687    
  !avg stdev:      2.857641       3.193110       1.332610      0.6160206      0.2479300    3.618900       3.885174       1.423129       1.247114    
#endif

#if 0
  !-----------------------------------------------------------------------
  !3x3; 2 tiles per grid; 1 ensemble per tile
  !-----------------------------------------------------------------------
  ! grid           1 tile            1  fgrd =   0.7153996
  ! grid           1 tile            1  pens =    1.000000
  ! grid           1 tile            2  fgrd =   0.2846004
  ! grid           1 tile            2  pens =    1.000000
  ! grid           2 tile            3  fgrd =   0.3938053
  ! grid           2 tile            3  pens =    1.000000
  ! grid           2 tile            4  fgrd =   0.6061947
  ! grid           2 tile            4  pens =    1.000000
  ! grid           3 tile            5  fgrd =   0.7767442
  ! grid           3 tile            5  pens =    1.000000
  ! grid           3 tile            6  fgrd =   0.2232558
  ! grid           3 tile            6  pens =    1.000000
  ! grid           4 tile            7  fgrd =   0.6829680
  ! grid           4 tile            7  pens =    1.000000
  ! grid           4 tile            8  fgrd =   0.3170320
  ! grid           4 tile            8  pens =    1.000000
  ! grid           5 tile            9  fgrd =   0.4691358
  ! grid           5 tile            9  pens =    1.000000
  ! grid           5 tile           10  fgrd =   0.5308642
  ! grid           5 tile           10  pens =    1.000000
  ! grid           6 tile           11  fgrd =   0.6356590
  ! grid           6 tile           11  pens =    1.000000
  ! grid           6 tile           12  fgrd =   0.3643411
  ! grid           6 tile           12  pens =    1.000000
  ! grid           7 tile           13  fgrd =   0.4098039
  ! grid           7 tile           13  pens =    1.000000
  ! grid           7 tile           14  fgrd =   0.5901961
  ! grid           7 tile           14  pens =    1.000000
  ! grid           8 tile           15  fgrd =   0.3782696
  ! grid           8 tile           15  pens =    1.000000
  ! grid           8 tile           16  fgrd =   0.6217304
  ! grid           8 tile           16  pens =    1.000000
  ! grid           9 tile           17  fgrd =   0.2273731
  ! grid           9 tile           17  pens =    1.000000
  ! grid           9 tile           18  fgrd =   0.7726269
  ! grid           9 tile           18  pens =    1.000000

  !First output at min 15
  array(1,:)=(/4.716996, -5.8574688E-02, -4.946619, -4.528573, 4.050246, 0.3562459, 1.049067, 9.616941, 9.012380, 9.418849, 9.940899, 6.408007, -1.736115, 5.853973, -7.550773, -2.725465, 1.651574, 6.570506/)

  !Second output at hour 1
  array(2,:)=(/6.697559, -3.812158, 7.377232, -6.422610, -3.549520, -4.753593, -7.389554, -4.095519, 9.354531, -2.984151, 2.352701, -6.535456, 0.7984526, -8.943732, 6.086356, -2.954750, -4.079035, -6.946579/)
  array(3,:)=(/-7.382446, -1.403471, -7.977238, -5.001271, 5.461119, -6.588569, -4.501272, -4.737327, -1.895595, -2.482169, 1.397863, 0.5272363, -8.027037, 9.894200, -4.217344, -7.860428, -3.925225, 6.145012/)
  array(4,:)=(/-9.781530, 9.290967, 8.008259, -3.773006, -2.737287, 2.721999, 8.338540, 1.821654, -3.868184, -2.530884, 6.939022, -9.691016, 4.223316, 9.007157, 9.105985, -8.339506, 6.217335, -7.065415/)
  !instantaneous: -4.353490      0.8665185      -1.518470       6.272479      -3.158259    0.8800159       7.046721      -1.740407      -4.045275
  !accumulated:  -6.327828      -6.294862      -2.565850      -4.648870      -2.560879    1.075054       4.645380      -7.757542      -6.484540
  !averaged:    -2.109276      -2.098287     -0.8552833      -1.549623     -0.8536262    0.3583512       1.548460      -2.585847      -2.161513
  !min:           -9.781530      -7.977238      -6.588569      -7.389554      -3.868184     -9.691016      -8.943732      -8.339506      -7.065415
  !max:            9.290967       8.008259       5.461119       8.338540       9.354531      6.939022        9.894200       9.105985       6.217335
  !inst mean:     -4.353490      0.8665185      -1.518470       6.272479      -3.158259    0.8800159       7.046721      -1.740407      -4.045275    
  !inst stdev:     8.605967       5.756238       2.273402       3.032437      0.6673750    8.003120       2.352680       8.460283       5.567272    
  !accum mean:    -6.327828      -6.294863      -2.565850      -4.648870      -2.560879    1.075055       4.645380      -7.757541      -6.484540    
  !accum stdev:    6.561588       11.04470       3.245841       1.609498       5.782928    12.69949       6.375117       14.61155       2.548368    
  !avg mean:      -2.109276      -2.098288     -0.8552831      -1.549623     -0.8536263    0.3583513       1.548460      -2.585847      -2.161514    
  !avg stdev:      2.187196       3.681567       1.081947      0.5364993       1.927643    4.233162       2.125039       4.870517      0.8494560    

  !Third output at hour 2
  array(5,:)=(/3.893483, -5.318944, -3.282447, 4.280159, 0.6406029, -5.399737, 9.492588, 2.222070, -3.001371, -8.529298, 9.347016, -8.244946, 8.028025, 1.886872, -2.056841, -5.433362, -2.393033, 0.5268470/)
  array(6,:)=(/2.292842, 1.003245, 3.493016, 8.131827, 0.7645558, -3.0272238E-02, -4.130377, -4.099755, 3.070868, -9.546155, -9.099010, -3.588018, -7.537397, 1.582848, 8.247005, 2.512011, -5.205674, -4.737566/)
  array(7,:)=(/7.338583, 4.381160, -8.673751, 6.1191890E-02, -5.017724, -7.243008, -6.492382, -1.514535, -3.883759, 2.662015, 7.436269, 1.580206, -5.054608, 9.078156, -1.479063, 6.945909, 8.579399, -6.499947/)
  array(8,:)=(/3.083600, -6.318086, -5.699180, -1.090655, 0.3126619, -1.287766, -9.275128, -2.084915, -4.124979, 9.457637, 9.837383, -1.176786, -7.695197, 6.274336, 6.044065, -4.654425, -7.474986, -5.760541/)
  !instantaneous: 0.4078764      -2.905517     -4.4642910E-02  -6.995601       3.085545    5.824470      0.5495665     -0.6075115      -6.150360
  !accumulated:  10.10222       1.322812      -5.680007      -8.842914      -6.886304    6.973547       6.084937       3.676745      -14.20272
  !averaged:     2.525555      0.3307029      -1.420002      -2.210728      -1.721576    1.743387       1.521234      0.9191862      -3.550681
  !min:          -6.318086     -8.673751       -7.243008      -9.275128      -9.546155   -9.099010     -7.695197       -5.433362      -7.474986
  !max:           7.338583     8.131827        0.7645558      9.492588        9.457637    9.837383      9.078156        8.247005       8.579399
  !inst mean:     0.4078764      -2.905517     -4.4642910E-02  -6.995601       3.085545    5.824470      0.5495665     -0.6075115      -6.150360    
  !inst stdev:    4.242266       2.251691      0.6664637       3.345750       6.778357    5.300513       6.870179       5.188291      0.7185848    
  !accum mean:    10.10222       1.322811      -5.680007      -8.842914      -6.886303    6.973547       6.084938       3.676745      -14.20272    
  !accum stdev:   10.31549       12.48104       4.439494       2.293173      0.9898287    13.93262       15.28575       5.521233       4.181678    
  !avg mean:      2.525555      0.3307027      -1.420002      -2.210728      -1.721576    1.743387       1.521234      0.9191863      -3.550681    
  !avg stdev:     2.578873       3.120260       1.109873      0.5732933      0.2474572    3.483155       3.821436       1.380308       1.045420    
#endif

#if 1
  !-----------------------------------------------------------------------
  !3x3; 2 tiles per grid; 2 ensembles per tile
  !-----------------------------------------------------------------------
  ! grid           1 tile            1  fgrd =   0.7153996
  ! grid           1 tile            1  pens =   0.5000000
  ! grid           1 tile            2  fgrd =   0.7153996
  ! grid           1 tile            2  pens =   0.5000000
  ! grid           1 tile            3  fgrd =   0.2846004
  ! grid           1 tile            3  pens =   0.5000000
  ! grid           1 tile            4  fgrd =   0.2846004
  ! grid           1 tile            4  pens =   0.5000000
  ! grid           2 tile            5  fgrd =   0.3938053
  ! grid           2 tile            5  pens =   0.5000000
  ! grid           2 tile            6  fgrd =   0.3938053
  ! grid           2 tile            6  pens =   0.5000000
  ! grid           2 tile            7  fgrd =   0.6061947
  ! grid           2 tile            7  pens =   0.5000000
  ! grid           2 tile            8  fgrd =   0.6061947
  ! grid           2 tile            8  pens =   0.5000000
  ! grid           3 tile            9  fgrd =   0.7767442
  ! grid           3 tile            9  pens =   0.5000000
  ! grid           3 tile           10  fgrd =   0.7767442
  ! grid           3 tile           10  pens =   0.5000000
  ! grid           3 tile           11  fgrd =   0.2232558
  ! grid           3 tile           11  pens =   0.5000000
  ! grid           3 tile           12  fgrd =   0.2232558
  ! grid           3 tile           12  pens =   0.5000000
  ! grid           4 tile           13  fgrd =   0.6829680
  ! grid           4 tile           13  pens =   0.5000000
  ! grid           4 tile           14  fgrd =   0.6829680
  ! grid           4 tile           14  pens =   0.5000000
  ! grid           4 tile           15  fgrd =   0.3170320
  ! grid           4 tile           15  pens =   0.5000000
  ! grid           4 tile           16  fgrd =   0.3170320
  ! grid           4 tile           16  pens =   0.5000000
  ! grid           5 tile           17  fgrd =   0.4691358
  ! grid           5 tile           17  pens =   0.5000000
  ! grid           5 tile           18  fgrd =   0.4691358
  ! grid           5 tile           18  pens =   0.5000000
  ! grid           5 tile           19  fgrd =   0.5308642
  ! grid           5 tile           19  pens =   0.5000000
  ! grid           5 tile           20  fgrd =   0.5308642
  ! grid           5 tile           20  pens =   0.5000000
  ! grid           6 tile           21  fgrd =   0.6356590
  ! grid           6 tile           21  pens =   0.5000000
  ! grid           6 tile           22  fgrd =   0.6356590
  ! grid           6 tile           22  pens =   0.5000000
  ! grid           6 tile           23  fgrd =   0.3643411
  ! grid           6 tile           23  pens =   0.5000000
  ! grid           6 tile           24  fgrd =   0.3643411
  ! grid           6 tile           24  pens =   0.5000000
  ! grid           7 tile           25  fgrd =   0.4098039
  ! grid           7 tile           25  pens =   0.5000000
  ! grid           7 tile           26  fgrd =   0.4098039
  ! grid           7 tile           26  pens =   0.5000000
  ! grid           7 tile           27  fgrd =   0.5901961
  ! grid           7 tile           27  pens =   0.5000000
  ! grid           7 tile           28  fgrd =   0.5901961
  ! grid           7 tile           28  pens =   0.5000000
  ! grid           8 tile           29  fgrd =   0.3782696
  ! grid           8 tile           29  pens =   0.5000000
  ! grid           8 tile           30  fgrd =   0.3782696
  ! grid           8 tile           30  pens =   0.5000000
  ! grid           8 tile           31  fgrd =   0.6217304
  ! grid           8 tile           31  pens =   0.5000000
  ! grid           8 tile           32  fgrd =   0.6217304
  ! grid           8 tile           32  pens =   0.5000000
  ! grid           9 tile           33  fgrd =   0.2273731
  ! grid           9 tile           33  pens =   0.5000000
  ! grid           9 tile           34  fgrd =   0.2273731
  ! grid           9 tile           34  pens =   0.5000000
  ! grid           9 tile           35  fgrd =   0.7726269
  ! grid           9 tile           35  pens =   0.5000000
  ! grid           9 tile           36  fgrd =   0.7726269
  ! grid           9 tile           36  pens =   0.5000000

  !First output at min 15
  array(1,:)=(/4.716996, -5.8574688E-02, -4.946619, -4.528573, 4.050246, 0.3562459, 1.049067, 9.616941, 9.012380, 9.418849, 9.940899, 6.408007, -1.736115, 5.853973, -7.550773, -2.725465, 1.651574, 6.570506, 6.697559, -3.812158, 7.377232, -6.422610, -3.549520, -4.753593, -7.389554, -4.095519, 9.354531, -2.984151, 2.352701, -6.535456, 0.7984526, -8.943732, 6.086356, -2.954750, -4.079035, -6.946579/)

  !Second output at hour 1
  array(2,:)=(/-7.382446, -1.403471, -7.977238, -5.001271, 5.461119, -6.588569, -4.501272, -4.737327, -1.895595, -2.482169, 1.397863, 0.5272363, -8.027037, 9.894200, -4.217344, -7.860428, -3.925225, 6.145012, -9.781530, 9.290967, 8.008259, -3.773006, -2.737287, 2.721999, 8.338540, 1.821654, -3.868184, -2.530884, 6.939022, -9.691016, 4.223316, 9.007157, 9.105985, -8.339506, 6.217335, -7.065415/)
  array(3,:)=(/3.893483, -5.318944, -3.282447, 4.280159, 0.6406029, -5.399737, 9.492588, 2.222070, -3.001371, -8.529298, 9.347016, -8.244946, 8.028025, 1.886872, -2.056841, -5.433362, -2.393033, 0.5268470, 2.292842, 1.003245, 3.493016, 8.131827, 0.7645558, -3.0272238E-02, -4.130377, -4.099755, 3.070868, -9.546155, -9.099010, -3.588018, -7.537397, 1.582848, 8.247005, 2.512011, -5.205674, -4.737566/)
  array(4,:)=(/7.338583, 4.381160, -8.673751, 6.1191890E-02, -5.017724, -7.243008, -6.492382, -1.514535, -3.883759, 2.662015, 7.436269, 1.580206, -5.054608, 9.078156, -1.479063, 6.945909, 8.579399, -6.499947, 3.083600, -6.318086, -5.699180, -1.090655, 0.3126619, -1.287766, -9.275128, -2.084915, -4.124979, 9.457637, 9.837383, -1.176786, -7.695197, 6.274336, 6.044065, -4.654425, -7.474986, -5.760541/)
  !instantaneous: 2.966581      -4.841046      0.5319989       2.240560     -0.3707637    -2.335645     -0.7540381       1.196324      -4.955079    
  !accumulated:  -2.390897      -5.249643      -5.308475       3.162108      0.4568557    2.836141      -4.157762      0.5380988      -7.813617    
  !averaged:   -0.7969657      -1.749881      -1.769492       1.054036      0.1522852    0.9453802      -1.385920      0.1793663      -2.604539    
  !min:         -8.673751   -7.243008   -8.529298   -8.027037   -9.781530   -5.699180   -9.546155   -9.691016    -8.339506
  !max:          7.338583    9.492588    9.347016    9.894200    9.290967     8.131827   9.457637    9.837383     9.105985
  !inst mean:   2.966581      -4.841046      0.5319989       2.240560     -0.3707637    -2.335645     -0.7540381       1.196324      -4.955079    
  !inst stdev:  5.294854       2.307088       3.844308       6.312026       6.337030    2.359247       7.026112       6.912323       4.058066    
  !accum mean: -2.390897      -5.249643      -5.308476       3.162108      0.4568557    2.836141      -4.157762      0.5380998      -7.813617    
  !accum stdev: 7.625556       7.148480       8.363175       12.77579       3.216291    2.628998       1.024142       13.28428       12.20573    
  !avg mean:   -0.7969656      -1.749881      -1.769492       1.054036      0.1522852    0.9453803      -1.385921      0.1793667      -2.604539    
  !avg stdev:   2.541852       2.382827       2.787725       4.258596       1.072097    0.8763326      0.3413807       4.428095       4.068575    

  !Third output at hour 2
  array(5,:)=(/1.512158, -8.205102, -2.326720, 6.680842, 6.766158, -4.824292, -0.6552086, 8.798944, 2.914175, 4.702343, 0.6865306, -9.221561, 3.526052, -6.942091, 0.7717164, -0.1421686, -5.030767, -1.006695, -3.380867, 5.706842, 3.186790, -1.400715, 9.258888, 5.078972, 6.693261, -5.327023, -2.919460, -9.476124, -5.032732, 5.105360, -4.468130, 0.1967068, 6.877312, -3.332381, -9.993605, -6.880735/)
  array(6,:)=(/-5.034120, -2.030401, 2.9046787E-02, 8.619015, 0.6473411, 2.842370, 1.923487, -5.885015, -4.387562, -9.323095, 9.576748, 9.665579, -5.410891, -5.991014, 3.737068, -1.512193, -6.807520, -9.950296, 8.058430, -4.378022, 6.450455, -9.427858, 1.836568, 3.195936, 3.863342, -1.176814, -9.614834, 0.1768892, 8.809194, 8.246654, 0.5746562, 6.527842, -6.950505, 1.441790, -4.565605, -0.1655554/)
  array(7,:)=(/-1.387385, 2.734760, -4.504713, 7.868738, 5.543264, 3.244931, -1.023396, -6.464956, 9.662812, 3.808145, 9.492757, 9.715104, -2.002011, -2.867249, -6.383791, -1.465850, -8.876604, 3.843547, -0.3149332, -3.985072, 1.818878, 1.528318, 1.592421, -3.959540, -9.640607, -0.1533833, 1.933727, -6.961665, -9.162514, -5.847863, -7.875674, -6.204535, -1.622068, -2.417793, -4.456558, -5.033423/)
  array(8,:)=(/-7.900800, -4.462110, -5.066877, 2.484161, 9.444094, -9.529427, -7.140241, 6.484406, 9.623430, -4.421771, -3.465148, -9.701500, -4.358383, 0.1882478, -8.888058, -4.864982, -0.1735505, -4.689309, 5.782395, 8.807663, -9.307220, 5.118613, -9.523894, -4.506863, 1.247825, -0.2931313, 2.083664, -7.577814, 5.102167, 8.240275, -2.010587, -0.4206751, 8.820971, 4.747029, 4.894506, 1.360849/)
  !instantaneous: -4.789732     -0.2155843      0.5504140      -3.604111       2.731999    -3.887254      -1.425694       1.767726       3.959027    
  !accumulated:  -6.899904       1.582243       6.754731      -11.11880      -3.342708    -0.1045657      -10.52885      -1.328632      -8.736110    
  !averaged:   -1.724976      0.3955607       1.688683      -2.779700     -0.8356769    -2.6141435E-02  -2.632212     -0.3321581      -2.184027    
  !min:          -8.205102   -9.529427   -9.701500   -8.888058   -9.950296   -9.523894   -9.640607   -9.162514   -9.993605
  !max:           8.619015    9.444094    9.715104    3.737068     8.807663    9.258888    6.693261   8.809194   8.820971
  !inst mean     -4.789732     -0.2155843      0.5504140      -3.604111       2.731999    -3.887254      -1.425694       1.767726       3.959027    
  !inst stdev     3.322716       7.974552       7.423239       3.127824       5.212215    6.400948       4.065803       3.994127       2.388276    
  !accum mean    -6.899903       1.582243       6.754730      -11.11880      -3.342708    -0.1045645      -10.52885      -1.328633      -8.736109    
  !accum stdev   13.26537       11.25949       10.85784       3.360188       12.69699    2.974003       9.439612       10.20610       7.134050    
  !avg mean      -1.724976      0.3955609       1.688682      -2.779700     -0.8356769    -2.6141133E-02  -2.632212     -0.3321581      -2.184027    
  !avg stdev      3.316342       2.814873       2.714461      0.8400470       3.174249    0.7435006       2.359903       2.551526       1.783513    
#endif

  !-----------------------------------------------------------------------
  ! diagnose output
  !-----------------------------------------------------------------------
  do t = 1,LIS_rc%ntiles(n)
     ! instantaneous; min/max; std
     call LIS_diagnoseOutputVar(n,t,LIS_MOC_SWNET,value=array(ts,t),&
                                vlevel=1,unit="W/m2",direction="DN")
     ! time-averaged; std
     call LIS_diagnoseOutputVar(n,t,LIS_MOC_LWNET,value=array(ts,t),&
                                vlevel=1,unit="W/m2",direction="DN")
     ! instantaneous and time-averaged; direction flip; min/max
     call LIS_diagnoseOutputVar(n,t,LIS_MOC_QLE  ,value=array(ts,t),&
                                vlevel=1,unit="W/m2",direction="DN")
     ! accumulated; direction flip; min/max; std
     call LIS_diagnoseOutputVar(n,t,LIS_MOC_QH   ,value=array(ts,t),&
                                vlevel=1,unit="W/m2",direction="DN")
                                !minValue=-1.0*ts, maxValue=1.0*ts)
     ! instantaneous and time-averaged; std
     call LIS_diagnoseOutputVar(n,t,LIS_MOC_QG   ,value=array(ts,t),&
                                vlevel=1,unit="W/m2",direction="DN")
  enddo
end subroutine template_main
